
#include <string.h>
#include <stdio.h>
#include <malloc.h>
#include <ps2gs.h>
#include "loaders.h"

void init_image(image_t *im)
{
	memset(im, 0, sizeof(image_t));
}

unsigned int swapBits3and4(unsigned int value) // works on 32bit value
{
	return (value&0xe7e7e7e7) | ((value&0x08080808)<<1) | ((value&0x10101010)>>1);
}


int load_image_pcx(char *filename, image_t *im)
{
	int ttmp;
	struct pcxheader {
		unsigned char manufacturer, version, encoding, bpp;
		short int xmin, ymin, xmax, ymax;
		unsigned char other[116];
	} header;
	FILE *im_file=fopen(filename, "rb");
	unsigned char howmany, databyte;
	int xtmp=0, ytmp=0;

	if (im_file==NULL)
	{
		printf("Cannot open image \"%s\"\n", filename);
		init_image(im);
		return 1;
	}

	fseek(im_file, 0, SEEK_SET);
	fread(&header, 1, 128, im_file);

	fseek(im_file, -769, SEEK_END);
	fread(&databyte, 1, 1, im_file);
	if (databyte==12)
	{
		im->clut_data=(unsigned char*)memalign(16, 256*4);
		for (ttmp=0; ttmp<256*4; ttmp+=4)
			fread(im->clut_data+ttmp, 1, 3, im_file);
	}
	else return 3;

	if ( (header.version!=5) || (header.bpp!=8) || (databyte!=12) )
	{
		printf("PCX image \"%s\" has unsupported format.\n", filename);
		fclose(im_file);
		init_image(im);
		return 2;
	}

	im->texture.w=header.xmax-header.xmin+1;
	im->texture.h=header.ymax-header.ymin+1;

	im->text_data=(unsigned char*)memalign(16, im->texture.w*im->texture.h);
	fseek(im_file, 128, SEEK_SET);

	while (ytmp<im->texture.h)
	{
		int c;
		fread (&databyte, 1, 1, im_file);
		howmany=1;
		if ((databyte&0xC0)==0xC0)
		{
			howmany = databyte & 0x3F;
			fread (&databyte, 1, 1, im_file);
		}
		for (c=1; c<=howmany; c++)
		{
			im->text_data[ytmp*im->texture.w+xtmp]=databyte;
			if (++xtmp>=im->texture.w) { xtmp=0; ytmp++; }
		}
	}

	fclose(im_file);

	// swap bits 3 and 4 to get equivalent to CLUT adjustment
	for (ttmp=0; ttmp<(im->texture.w*im->texture.h/4); ttmp++)
	{
		((unsigned int*)im->text_data)[ttmp]=swapBits3and4(((unsigned int*)im->text_data)[ttmp]);
	}

	im->texture.ptr=im->text_data;
	im->clut.ptr=im->clut_data;

	im->texture.psm=PS2_GS_PSMT8;
	if (im->texture.w>64) im->texture.fbw=im->texture.w/64; else im->texture.fbw=1;
	im->clut.psm=PS2_GS_PSMCT32;
	im->clut.h=im->clut.w=16;
	im->clut.fbw=1;

	ttmp=2; im->tw=1; while (ttmp<im->texture.w) { ttmp*=2; im->tw++; }
	ttmp=2; im->th=1; while (ttmp<im->texture.h) { ttmp*=2; im->th++; }
	
	return 0;
}

void free_image(image_t *im)
{
	if (im->text_data!=NULL) free(im->text_data);
	im->text_data=NULL;
	if (im->clut_data!=NULL) free(im->clut_data);
	im->clut_data=NULL;
}
