
#include <unistd.h>
#include <stdio.h>
#include <linux/ps2/ee.h>
#include "packet.h"



/////////////////////////////////////////////
// VIF PACKET
/////////////////////////////////////////////

void vifpacket_align(vifpacket_t *vpkt, u_int bit, u_int pos)
{
	u_int const mask = 0xffffffff >> (32 - ((bit + 2) & 0x1f));
	u_int target = ((u_int)vpkt->current & ~mask) + pos * sizeof(u_int);
	if(target < (u_int)vpkt->current){ target += mask + 1; }
	while(target > (u_int)vpkt->current)
	{
		*vpkt->current++ = 0;
	}
}

void vifpacket_init(vifpacket_t *vpkt, u_long128 *buffer)
{
	vpkt->base=buffer;
	vpkt->current=(u_int*)vpkt->base;
	vpkt->dmatag=NULL;
}

void vifpacket_reset(vifpacket_t *vpkt)
{
	vpkt->current=(u_int*)vpkt->base;
	vpkt->dmatag=NULL;
}

u_long128 *vifpacket_terminate(vifpacket_t *vpkt)
{
	u_long128 *dma_current=align128(vpkt->current);
	if (vpkt->dmatag!=NULL)
	{
		u_int count=((u_int)dma_current-(u_int)vpkt->dmatag)/16-1;
		((u_int*)vpkt->dmatag)[0]+=count;
	}
	vpkt->current=(u_int*)dma_current;

	vpkt->dmatag=NULL;
	return dma_current;
}


int vifpacket_size(vifpacket_t *vpkt)
{
	return ((u_int)vpkt->current - (u_int)vpkt->base) / sizeof(u_long128);
}

void vifpacket_cnt(vifpacket_t *vpkt, u_int flag)
{
	vpkt->dmatag = vifpacket_terminate(vpkt);
	*vpkt->current++ = (1 << 28) | flag;
	*vpkt->current++ = 0;
	*vpkt->current++ = 0;
	*vpkt->current++ = 0;
	vpkt->vifcodes = NULL;
}

void vifpacket_next(vifpacket_t *vpkt, u_long128 *next, u_int flag){
	vpkt->dmatag = vifpacket_terminate(vpkt);
	*vpkt->current++ = (2 << 28) | flag;
	*vpkt->current++ = CV2P(next);
	*vpkt->current++ = 0;
	*vpkt->current++ = 0;
	vpkt->vifcodes = NULL;
}


void vifpacket_end(vifpacket_t *vpkt, u_int flag)
{
	vpkt->dmatag = vifpacket_terminate(vpkt);
	*vpkt->current++ = (7 << 28) | flag;
	*vpkt->current++ = 0;
	*vpkt->current++ = 0;
	*vpkt->current++ = 0;
	vpkt->vifcodes = NULL;
}

void vifpacket_opendirectcode(vifpacket_t *vpkt, int stall)
{
	vifpacket_align(vpkt, 2, 3);
	vpkt->vifcodes = vpkt->current;
	*vpkt->current++ = ((stall ? 0x80 : 0) | 0x50) << 24;
}

void vifpacket_closedirectcode(vifpacket_t *vpkt)
{
	u_int const count = (vpkt->current - (vpkt->vifcodes + 1)) / (sizeof(u_long128) / sizeof(u_int));
	*vpkt->vifcodes += count;
	vpkt->vifcodes = NULL;
}

void vifpacket_addcode(vifpacket_t *vpkt, u_int code)
{
	*vpkt->current++ = code;
}

void vifpacket_openupkcode(vifpacket_t *vpkt, u_short vuaddr, u_int upkcmd, u_int cl, u_int wl)
{
	*vpkt->current++ = (0x01 << 24) | (wl << 8) | cl;
	vpkt->vifcodes = vpkt->current;
	*vpkt->current++ = (upkcmd << 24) | vuaddr;
	{
		u_int const clVal = (cl == 0) ? 256 : cl;
		u_int const wlVal = (wl == 0) ? 256 : wl;
		u_int len = (32 >> (upkcmd & 0x03)) * (((upkcmd >> 2) & 0x03) + 1);
		if(wlVal <= clVal)
		{
			vpkt->len = (1 << 16) | len;
		}
		else
		{
			len *= clVal;
			vpkt->len = (wlVal << 16) | len;
		}
	}
}

void vifpacket_closeupkcode(vifpacket_t *vpkt)
{
	u_int const bitcount = (vpkt->current - (vpkt->vifcodes + 1)) * sizeof(u_int) * 8;
	u_int const numlen = vpkt->len;
	u_int const len = numlen & 0x0000ffff;
	u_int const num = (numlen & 0xffff0000) * ( bitcount / len);
	*vpkt->vifcodes += num;
	vpkt->vifcodes = NULL;
}

void vifpacket_adddata(vifpacket_t *vpkt, u_int data)
{
  *vpkt->current++ = data;
}

void vifpacket_addupkdata64( vifpacket_t *vpkt, u_long64 data )
{
	*vpkt->current++ = (u_int)data;
	*vpkt->current++ = (u_int)(data >> 32);
}

void vifpacket_addupkdata64N(vifpacket_t *vpkt, u_long64* pData, u_int count)
{
	while(count--)
	{
		u_long64 const data = *pData++;
		*vpkt->current++ = (u_int)data;
		*vpkt->current++ = (u_int)(data >> 32);
	}
}

void vifpacket_addupkdata128( vifpacket_t *vpkt, u_long128 data )
{
	u_long64 const low = ((u_long64*)&data)[0];
	u_long64 const high = ((u_long64*)&data)[1];
	*vpkt->current++ = (u_int)low;
	*vpkt->current++ = (u_int)(low >> 32);
	*vpkt->current++ = (u_int)high;
	*vpkt->current++ = (u_int)(high >> 32);
}

void vifpacket_addupkdata128N(vifpacket_t *vpkt, u_long128* pData, u_int count)
{
	while(count--)
	{
		u_long64 const low = ((u_long64*)pData)[0];
		u_long64 const high = ((u_long64*)pData)[1];
		pData++;
		*vpkt->current++ = (u_int)low;
		*vpkt->current++ = (u_int)(low >> 32);
		*vpkt->current++ = (u_int)high;
		*vpkt->current++ = (u_int)(high >> 32);
	}
}






/////////////////////////////////////////////
// GIF PACKET
/////////////////////////////////////////////


void gifpacket_init(gifpacket_t *gpkt, u_long128 *buffer)
{
	gpkt->base=align128((u_int*)buffer);
	gpkt->current=(u_int*)gpkt->base;
	gpkt->dmatag=NULL;
}

void gifpacket_reset(gifpacket_t *gpkt)
{
	gpkt->current=(u_int*)gpkt->base;
	gpkt->dmatag=NULL;
}

u_long128 *gifpacket_terminate(gifpacket_t *gpkt)
{
	u_long128* const dma_current = align128(gpkt->current);
	if(gpkt->dmatag)
	{
		u_int const count = (dma_current - gpkt->dmatag) - 1;
		((u_int*)gpkt->dmatag)[0] += count;
	}
	gpkt->current = (u_int*)dma_current;
	gpkt->dmatag = NULL;
	return dma_current;
}

int  gifpacket_size(gifpacket_t *gpkt)
{
	return ((u_int)gpkt->current - (u_int)gpkt->base) / sizeof(u_long128);
}


void gifpacket_addgsdata(gifpacket_t *gpkt, u_long64 data)
{
	*((u_long64*)gpkt->current) = data;
	gpkt->current+=sizeof(u_long64)/sizeof(u_int);
}


void gifpacket_addgspacked(gifpacket_t* gpkt, u_long128 data)
{
	*((u_long128*)gpkt->current) = data;
	gpkt->current += sizeof(u_long128) / sizeof(u_int);
}

void gifpacket_dump(gifpacket_t *gpkt)
{
	int size=gifpacket_size(gpkt);
	int i;
	unsigned int *dat=(unsigned int *)gpkt->base;

	for (i=0; i<size; i++)
	{
		printf("-gifpacket: 0x%.8x 0x%.8x 0x%.8x 0x%.8x\n", dat[0], dat[1], dat[2], dat[3]);
		dat+=4;
	}
}




